#include "wBLRegion.h"
#include <iostream>
#include <vector>
#include <cmath>

using namespace tbox;
using namespace dart;


BLRegion::BLRegion()
{
  std::cout << "Coucou de BLRegion\n" << std::endl;

  
} // end BLRegion


BLRegion::~BLRegion()
{
    std::cout << "~BLRegion()\n";
} // end ~BLRegion

void BLRegion::SetMesh(std::vector<double> locM, std::vector<double> globM)
{
  LocMarkers=locM;
  GlobMarkers=globM;
  nMarkers = locM.size();

  /* Resize all variables */

  U.resize(nVar * nMarkers);
  Ret.resize(nMarkers);
  Cd.resize(nMarkers);
  Cf.resize(nMarkers);
  Hstar.resize(nMarkers);
  Hstar2.resize(nMarkers);
  Hk.resize(nMarkers);
  Cteq.resize(nMarkers);
  Us.resize(nMarkers);
  DeltaStar.resize(nMarkers);
  Delta.resize(nMarkers);
  Me.resize(nMarkers);
  Rhoe.resize(nMarkers);
  VtExt.resize(nMarkers);
  DeltaStarExt.resize(nMarkers);
  XxExt.resize(nMarkers);
  ReExt.resize(nMarkers);
  Regime.resize(nMarkers);
} // End SetMesh

void BLRegion::SetInvBC(std::vector<double> _Ue,
                        std::vector<double> _Me,
                        std::vector<double> _Rhoe)
{
  VtExt = _Ue;
  Me = _Me;
  Rhoe = _Rhoe;
}// End SetInvBC

void BLRegion::SetStagBC(double Re)
{
  double dv0 = (VtExt[1] - VtExt[0])/(LocMarkers[1] - LocMarkers[0]);

  if (dv0 > 0){
    U[0] = sqrt(0.075/(Re*dv0)); // Theta
  } // End if
  else{
    U[0] = 1e-6; // Theta
  } // end else
  U[1] = 2.23; // H
  U[2] = 0; // N
  U[3] = VtExt[0]; // ue
  U[4] = 0; // Ctau
  
  Ret[0] = U[3]*U[0]*Re;

  if (Ret[0] < 1){
    Ret[0] = 0;
    U[3] = Ret[0]/(U[0]*Re);
  } // End if

  LaminarClosures(0);
} // End SetStagBC

unsigned int BLRegion::GetnVar(){return nVar;}
size_t BLRegion::GetnMarkers(){return nMarkers;}
double BLRegion::GetLocMarkers(size_t iPoint){return LocMarkers[iPoint];}
double BLRegion::GetGlobMarkers(size_t iPoint){return GlobMarkers[iPoint];}
double BLRegion::GetVtExt(size_t iPoint){return VtExt[iPoint];}
double BLRegion::GetMe(size_t iPoint){return Me[iPoint];}
double BLRegion::GetRhoe(size_t iPoint){return Rhoe[iPoint];}
double BLRegion::GetXxExt(size_t iPoint){return XxExt[iPoint];}
double BLRegion::GetDeltaStarExt(size_t iPoint){return DeltaStarExt[iPoint];}

void BLRegion::PrintSolution(size_t iPoint)
{
  std::cout << "Point " << iPoint << std::endl;
  std::cout << "Theta " << U[iPoint * nVar + 0] << " \n" << std::endl;
  std::cout << "H " << U[iPoint * nVar + 1] << " \n" << std::endl;
  std::cout << "N " << U[iPoint * nVar + 2] << " \n" << std::endl;
  std::cout << "ue " << U[iPoint * nVar + 3] << " \n" << std::endl;
  std::cout << "Ct " << U[iPoint * nVar + 4] << " \n" << std::endl;
  }

void BLRegion::LaminarClosures(size_t iPoint)
{
  double theta = U[iPoint*nVar+0];
  double H = U[iPoint*nVar+1];
  double ret = Ret[iPoint];
  double me = Me[iPoint];
  double hk = (H - 0.29*me*me)/(1+0.113*me*me); // Kinematic shape factor
  if(name == "airfoil"){
      hk = std::max(hk, 1.02);
  }
  else{
      hk = std::max(hk, 1.00005);
  }
  hk = std::min(hk,6.0);
  double hstar2 = (0.064/(hk-0.8)+0.251)*me*me; // Density shape parameter
  double delta = std::min((3.15+ H + (1.72/(hk-1)))*theta, 12*theta);
  double hks = hk - 4.35;
  double hstar;
  if(hk <= 4.35){
      double dens = hk + 1;
      hstar = 0.0111*hks*hks/dens - 0.0278*hks*hks*hks/dens + 1.528 -0.0002*(hks*hk)*(hks*hk);
  }
  else if(hk > 4.35){
      hstar = 1.528 + 0.015*hks*hks/hk;
  }
  hstar = (hstar + 0.028*me*me)/(1+0.014*me*me); // Whitfield's minor additional compressibility correction

  /* Cf */

  double cf;
  if(hk < 5.5){
      double tmp = (5.5-hk)*(5.5-hk)*(5.5-hk) / (hk+1);
      cf = (-0.07 + 0.0727*tmp)/ret;
  }
  else if(hk >= 5.5){
      double tmp = 1 - 1/(hk-4.5);
      cf = (-0.07 + 0.015*tmp*tmp) /ret;
  }

  /* Cd */

  double cd;
  if(hk < 4){
      cd = ( 0.00205*pow(4.0-hk, 5.5) + 0.207 ) * (hstar/(2*ret));
  }
  else if(hk >= 4){
      double hkCd = (hk-4)*(hk-4);
      double denCd = 1+0.02*hkCd;
      cd = ( -0.0016*hkCd/denCd + 0.207) * (hstar/(2*ret));
  }
  if (name == "wake"){
      cd = 2*(1.10*(1.0-1.0/hk)*(1.0-1.0/hk)/hk)* (hstar/(2*ret));
      cf = 0;
  }
  Cd[iPoint] = cd;
  Cf[iPoint] = cf;
  Delta[iPoint] = delta;
  Hstar[iPoint] = hstar;
  Hstar2[iPoint] = hstar2;
  Hk[iPoint] = hk;
  Cteq[iPoint] = 0;
  Us[iPoint] = 0;
}

